/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/ 
module key_scan(
    clk    ,
    rst_n  ,
    key_col,
    key_row,
    key_out,
    key_vld
    );

    parameter       DATA_W      =   4           ;
    parameter       TIME_20MS   =   1_000_000   ;

    input                   clk         ;
    input                   rst_n       ;
    input   [3:0]           key_col     ;

    output  [3:0]           key_row     ;
    output                  key_vld     ;
    output  [DATA_W-1:0]    key_out     ;

    reg     [3:0]           key_row     ;
    reg                     key_vld     ;
    reg     [DATA_W-1:0]    key_out     ;

    reg     [3:0]           key_col_ff0 ;
    reg     [3:0]           key_col_ff1 ;


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_col_ff0 <= 4'b1111;
        key_col_ff1 <= 4'b1111;
    end
    else begin
        key_col_ff0 <= key_col    ;
        key_col_ff1 <= key_col_ff0;
    end
end

reg         key_col_check;
always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_col_check <= 1'b0;
    end
    else if(key_col_ff1 !=4'hf && end_shake)begin
        key_col_check <= 1'b1;
    end
    else if(key_col_ff1==4'hf)begin
        key_col_check <= 1'b0;
    end
end

reg [ 21:0]  shake     ;
wire        add_shake ;
wire        end_shake ;
always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        shake <= 0; 
    end
    else if(add_shake) begin
        if(end_shake)
            shake <= 0; 
        else
            shake <= shake+1 ;
   end
end
assign add_shake = key_col_ff1 !=4'hf || key_row_check==1;
assign end_shake = add_shake  && shake == TIME_20MS-1 ;



reg     [1:0]   key_col_get     ;
always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_col_get <= 0;
    end
    else if(key_col_check) begin
        if(key_col_ff1==4'b1110)
            key_col_get <= 0;
        else if(key_col_ff1==4'b1101)
            key_col_get <= 1;
        else if(key_col_ff1==4'b1011)
            key_col_get <= 2;
        else if(key_col_ff1==4'b0111) 
            key_col_get <= 3;
    end
end

reg             key_row_check       ;
always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_row_check <= 0;
    end
    else if(key_col_check)begin
        key_row_check <= 1;
    end
    else if(key_vld)begin
        key_row_check <= 0;
    end
end

reg   [1:0]         row_index        ;
wire                add_row_index    ;
wire                end_row_index    ;
always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        row_index <= 0; 
    end
    else if(add_row_index) begin
        if(end_row_index)
            row_index <= 0; 
        else
            row_index <= row_index+1 ;
   end
end
assign add_row_index = key_row_check && end_shake;
assign end_row_index = add_row_index  && row_index == 4-1 ;



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_row = 4'b0;
    end
    else if(key_row_check)begin
        key_row = ~(4'b0001 << row_index);
    end
    else begin
        key_row = 4'b0;
    end
end



always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_vld <= 1'b0;
    end
    else if(key_row_check && key_col_ff1[key_col_get]==1'b0 && key_col_check==0 )begin 
        key_vld <= 1'b1;
    end
    else begin
        key_vld <= 1'b0;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        key_out <= 4'd0;
    end
    else if(key_vld)begin
        key_out <= {row_index,key_col_get};    
    end
    else begin
        key_out <= 4'd0;
    end
end

endmodule

