/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module seg_disp(
    clk         ,
    rst_n       ,
    data_in     ,
    key_en      ,
    segment     ,
    seg_sel      
);

parameter   ZERO           =   8'b0000_0011          ;
parameter   ONE            =   8'b1001_1111          ;
parameter   TWO            =   8'b0010_0101          ;
parameter   THREE          =   8'b0000_1101          ;
parameter   FOUR           =   8'b1001_1001          ;
parameter   FIVE           =   8'b0100_1001          ;
parameter   SIX            =   8'b0100_0001          ;
parameter   SEVEN          =   8'b0001_1111          ;
parameter   EIGHT          =   8'b0000_0001          ;
parameter   NINE           =   8'b0000_1001          ;

input                 clk             ;         
input                 rst_n           ;   
input    [4:0]        data_in         ;
input                 key_en          ;
output   [7:0 ]       segment         ; 
output   [1:0 ]       seg_sel         ;

reg      [7:0 ]       segment         ;
reg      [1:0 ]       seg_sel         ;
reg      [10:0]       delay           ;
reg      [1:0 ]       delay_time      ;
wire                  add_delay_time  ;
wire                  end_delay_time  ;
wire                  add_delay       ;
wire                  end_delay       ;
reg     [4:0 ]        segment_tmp     ;




always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        delay <= 0; 
    end
    else if(add_delay) begin
        if(end_delay)
            delay <= 0; 
        else
            delay <= delay+1 ;
   end
end
assign add_delay = 1;
assign end_delay = add_delay  && delay == 2000-1 ;



always @(posedge clk or negedge rst_n) begin 
    if (rst_n==0) begin
        delay_time <= 0; 
    end
    else if(add_delay_time) begin
        if(end_delay_time)
            delay_time <= 0;
        else
            delay_time <= delay_time+1 ;
   end
end
assign add_delay_time = end_delay;
assign end_delay_time = add_delay_time  && delay_time == 2-1 ;


reg     [3:0]       segment_data;
always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        segment_data <= 4'd0;
    end
    else if(key_en)begin
        segment_data <= data_in;
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        segment_tmp <= 4'd0;
    end
    else if(add_delay_time  && delay_time == 1-1)begin
        segment_tmp <= (segment_data+1)%10;
    end
    else if(end_delay_time)begin
        segment_tmp <= (segment_data+1)/10;
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        segment <= ZERO;
    end
    else begin
        case(segment_tmp)
            4'd0:segment <= ZERO;
            4'd1:segment <= ONE  ;
            4'd2:segment <= TWO  ;
            4'd3:segment <= THREE;
            4'd4:segment <= FOUR ;
            4'd5:segment <= FIVE ;
            4'd6:segment <= SIX  ;
            4'd7:segment <= SEVEN;
            4'd8:segment <= EIGHT;
            4'd9:segment <= NINE ;
            default:begin
                segment <= segment;
            end
        endcase
    end
end


always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        seg_sel <= 2'b11;
    end
    else begin
        seg_sel <= ~(2'b1<<delay_time);
    end
end


endmodule
